/* apps/xpgp_keyring.c */
/*
 * Copyright (C) 2005-2006 Mark Fernie (drbob@lunamutt.com)  
 * All rights reserved.
 */
/* ====================================================================
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit. (http://www.openssl.org/)"
 *
 * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    openssl-core@openssl.org.
 *
 * 5. Products derived from this software may not be called "OpenSSL"
 *    nor may "OpenSSL" appear in their names without prior written
 *    permission of the OpenSSL Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit (http://www.openssl.org/)"
 *
 * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 *
 */



#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef OPENSSL_NO_STDIO
#define APPS_WIN16
#endif
#include "apps.h"
#include <openssl/bio.h>
#include <openssl/asn1.h>
#include <openssl/err.h>
#include <openssl/bn.h>
#include <openssl/evp.h>
#include <openssl/x509.h>
#include <openssl/x509v3.h>
#include <openssl/objects.h>
#include <openssl/pem.h>

#include <openssl/xPGP.h>

#include <openssl/ssl.h>

#include <iostream>

const int MAX_PEOPLE = 14;

int main(int argc, char **argv)
{
	XPGP *certs[MAX_PEOPLE];
	EVP_PKEY *keys[MAX_PEOPLE];

	SSL_library_init();


	char name[128] = "Person";
	char *country = "virtua world";
	char *org = "Stuff";
	char *loc = "inside";
	char *nothing = "\0";
	int i,j;
        generate_xpgp(  "pgp_keytst_cert.pem", "pgp_keytst_pk.pem", "1234",
	                "KeyTest Cert", "ss@nowhere.com", "NwCom",
		        "wnyere", "Moseshire", "UK", 2048);

	fprintf(stderr, "Generated Test Certificate\n");

	XPGP_KEYRING *kr = xpgp_init("pgp_keytst_cert.pem", 
				"pgp_keytst_pk.pem", "1234");

	fprintf(stderr, "Created XPGP_KEYRING\n");

	/* now generate some random certificates.  */
	for(i = 0; i < MAX_PEOPLE; i++)
	{
		/* make the name */
		sprintf(&(name[6]), "%04d", i);

		fprintf(stderr, "Generating Certificates for: %s\n", name);

		generate_XPGP(&(certs[i]), &(keys[i]), name, nothing, 
			org, loc, nothing, country, 2048);
	}

	/* now sign them  */
	for(i = 0; i < MAX_PEOPLE; i++)
	{
		for(j = i+1; j < MAX_PEOPLE; j++)
		{
			fprintf(stderr, "Certificate [%d signing-> %d]\n", i, j);
			sign_XPGP(certs[j], certs[i], keys[i]);
		}
	}

	/* Add them in */
	for(i = 0; i < MAX_PEOPLE; i++)
	{
		/* display_XPGP(certs[i], stderr); */
		addCertificate(kr, certs[i]);
	}
	

	/* Test setup */
	checkKeyRing(kr);

	fprintf(stderr, "***************************************\n");
	fprintf(stderr, "Displaying - None Signed \n");
	fprintf(stderr, "***************************************\n");
	//DisplayStackTrust(kr);
	DisplayStacks(kr);


	/* sign some of them */
	/* lets sign number 3 */
	i = 4;
	fprintf(stderr, "Certificate [We Are signing-> %d]\n",i);
	sign_XPGP(certs[i], kr->houseKeyPublic, kr->houseKeyPrivate);


	/* Test setup */
	checkKeyRing(kr);

	fprintf(stderr, "***************************************\n");
	fprintf(stderr, "Displaying - 1 OwnSigned \n");
	fprintf(stderr, "***************************************\n");
	//DisplayStackTrust(kr);
	DisplayStacks(kr);


	/* more cert + signing */
	i = 6;
	fprintf(stderr, "Certificate [We Are signing-> %d]\n",i);
	sign_XPGP(certs[i], kr->houseKeyPublic, kr->houseKeyPrivate);

	XPGP_signer_trusted(kr, certs[i]);

	/* Test setup */
	checkKeyRing(kr);
	fprintf(stderr, "***************************************\n");
	fprintf(stderr, "Displaying - 1 OwnSigned / 1 Trusted\n");
	fprintf(stderr, "***************************************\n");
	//DisplayStackTrust(kr);
	DisplayStacks(kr);



	/* more cert + signing */
	i = 8;
	fprintf(stderr, "Certificate [We Are signing-> %d]\n",i);
	sign_XPGP(certs[i], kr->houseKeyPublic, kr->houseKeyPrivate);

	i = 12;
	fprintf(stderr, "Certificate [We Are signing-> %d]\n",i);
	sign_XPGP(certs[i], kr->houseKeyPublic, kr->houseKeyPrivate);

	i = 11;
	fprintf(stderr, "Certificate [We Are signing-> %d]\n",i);
	sign_XPGP(certs[i], kr->houseKeyPublic, kr->houseKeyPrivate);

	XPGP_signer_trusted(kr, certs[i]);

	/* Test setup */
	checkKeyRing(kr);
	fprintf(stderr, "***************************************\n");
	fprintf(stderr, "Displaying - 3 OwnSigned / 2 Trusted\n");
	fprintf(stderr, "***************************************\n");
	DisplayStackTrust(kr);
	DisplayStacks(kr);



	/* change parameter (security level) */

	/* Test setup */

	return 1;
}


