/* apps/xpgp_sign.c */
/*
 * Copyright (C) 2005-2006 Mark Fernie (drbob@lunamutt.com)  
 * All rights reserved.
 */
/* ====================================================================
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit. (http://www.openssl.org/)"
 *
 * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    openssl-core@openssl.org.
 *
 * 5. Products derived from this software may not be called "OpenSSL"
 *    nor may "OpenSSL" appear in their names without prior written
 *    permission of the OpenSSL Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit (http://www.openssl.org/)"
 *
 * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 *
 */


#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef OPENSSL_NO_STDIO
#define APPS_WIN16
#endif
#include "apps.h"
#include <openssl/bio.h>
#include <openssl/asn1.h>
#include <openssl/err.h>
#include <openssl/bn.h>
#include <openssl/evp.h>
#include <openssl/x509.h>
#include <openssl/x509v3.h>
#include <openssl/objects.h>
#include <openssl/pem.h>

#include <openssl/xPGP.h>

#include <openssl/ssl.h>

#include <iostream>

#include <unistd.h>

int usage(char *name)
{
	std::cerr << "Usage: " << name;
	std::cerr << " [-p <signer_password>]";
	std::cerr << std::endl;
	std::cerr << "\t -c <signer_cert.pem>";
	std::cerr << std::endl;
	std::cerr << "\t -k <signer_key.pem>";
	std::cerr << std::endl;
	std::cerr << "\t <signee_cert.pem>";
	std::cerr << std::endl;
	return 1;
}


int main(int argc, char **argv)
{
	SSL_library_init();
	int c = 0;
	std::string signing_cert;
	std::string signing_pkey;
	std::string signee_cert;
	std::string passwd_string;
	bool passwd_set = false;

	while(-1 != (c = getopt(argc, argv, "c:k:p:")))
	{
		switch(c)
		{
			case 'c':
				signing_cert = optarg;
				break;
			case 'k':
				signing_pkey = optarg;
				break;
			case 'p':
				passwd_set = true;
				passwd_string = optarg;
				break;
			default:
				std::cerr << "Unknown Option!" << std::endl;
				usage(argv[0]);
				exit(1);
		}
	}

	if (argc - optind < 1)
	{
		std::cerr << "Missing Target Cert" << std::endl;
		usage(argv[0]);
		exit(1);
	}

	signee_cert = argv[optind];

	std::cerr << "Signing    Cert: " << signing_cert  << std::endl;
	std::cerr << "Signing PrivKey: " << signing_pkey  << std::endl;
	std::cerr << "Signee (Target): " << signee_cert   << std::endl;
	//std::cerr << "Password       : " << passwd_string << std::endl;

	if (passwd_set)
	{
		sign_xpgp(signee_cert.c_str(), 
			signing_cert.c_str(),
			signing_pkey.c_str(),
			passwd_string.c_str());
	}
	else
	{
		sign_xpgp(signee_cert.c_str(), 
			signing_cert.c_str(),
			signing_pkey.c_str(), NULL);
	}
	return 1;
}


