/* crypto/asn1/t_xPGP.c */
/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
 * All rights reserved.
 *
 * This package is an SSL implementation written
 * by Eric Young (eay@cryptsoft.com).
 * The implementation was written so as to conform with Netscapes SSL.
 * 
 * This library is free for commercial and non-commercial use as long as
 * the following conditions are aheared to.  The following conditions
 * apply to all code found in this distribution, be it the RC4, RSA,
 * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
 * included with this distribution is covered by the same copyright terms
 * except that the holder is Tim Hudson (tjh@cryptsoft.com).
 * 
 * Copyright remains Eric Young's, and as such any Copyright notices in
 * the code are not to be removed.
 * If this package is used in a product, Eric Young should be given attribution
 * as the author of the parts of the library used.
 * This can be in the form of a textual message at program startup or
 * in documentation (online or textual) provided with the package.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    "This product includes cryptographic software written by
 *     Eric Young (eay@cryptsoft.com)"
 *    The word 'cryptographic' can be left out if the rouines from the library
 *    being used are not cryptographic related :-).
 * 4. If you include any Windows specific code (or a derivative thereof) from 
 *    the apps directory (application code) you must include an acknowledgement:
 *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
 * 
 * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 */

#include <stdio.h>
#include "cryptlib.h"
#include <openssl/buffer.h>
#include <openssl/bn.h>
#ifndef OPENSSL_NO_RSA
#include <openssl/rsa.h>
#endif
#ifndef OPENSSL_NO_DSA
#include <openssl/dsa.h>
#endif
#include <openssl/objects.h>
#include <openssl/x509.h>
#include <openssl/x509v3.h>
#include <openssl/xPGP.h>

/* Again this relies heavily on the X509 printing routines...
 * and isn't very pretty in places...
 *
 */

#ifndef OPENSSL_NO_FP_API
int XPGP_print_fp(FILE *fp, XPGP *x)
	{
	return XPGP_print_ex_fp(fp, x, XN_FLAG_COMPAT, X509_FLAG_COMPAT);
	}

int XPGP_print_ex_fp(FILE *fp, XPGP *x, unsigned long nmflag, unsigned long cflag)
        {
        BIO *b;
        int ret;

        if ((b=BIO_new(BIO_s_file())) == NULL)
		{
		X509err(X509_F_X509_PRINT_FP,ERR_R_BUF_LIB);
                return(0);
		}
        BIO_set_fp(b,fp,BIO_NOCLOSE);
        ret=XPGP_print_ex(b, x, nmflag, cflag);
        BIO_free(b);
        return(ret);
        }
#endif

int XPGP_print(BIO *bp, XPGP *x)
{
	return XPGP_print_ex(bp, x, XN_FLAG_COMPAT, X509_FLAG_COMPAT);
}

int XPGP_print_ex(BIO *bp, XPGP *x, unsigned long nmflags, unsigned long cflag)
	{
	long l;
	int ret=0,i;
	int nmindent = 0;
	EVP_PKEY *pkey=NULL;
	XPGP_SIGNATURE *nsig;

	if((nmflags & XN_FLAG_SEP_MASK) == XN_FLAG_SEP_MULTILINE) {
			nmindent = 12;
	}

	if(nmflags == X509_FLAG_COMPAT)
		nmindent = 16;

	if(!(cflag & XPGP_FLAG_NO_HEADER))
		{
		if (BIO_write(bp,"XPGP Certificate:\n",18) <= 0) goto err;
		if (BIO_write(bp,"    Data:\n",10) <= 0) goto err;
		}
	if(!(cflag & XPGP_FLAG_NO_VERSION))
		{
		l=XPGP_get_version(x);
		if (BIO_printf(bp,"%8sVersion: %lu (0x%lx)\n","",l+1,l) <= 0) goto err;
		}
	if(!(cflag & XPGP_FLAG_NO_VALIDITY))
		{
		if (BIO_write(bp,"        Validity\n",17) <= 0) goto err;
		if (BIO_write(bp,"            Not Before: ",24) <= 0) goto err;
		if (!ASN1_TIME_print(bp,XPGP_PUBKEY_get_notBefore(x->key))) goto err;
		if (BIO_write(bp,"\n            Not After : ",25) <= 0) goto err;
		if (!ASN1_TIME_print(bp,XPGP_PUBKEY_get_notAfter(x->key))) goto err;
		if (BIO_write(bp,"\n",1) <= 0) goto err;
		}
	if(!(cflag & XPGP_FLAG_NO_SUBJECT))
		{
		if (BIO_printf(bp,"        Subject: ") <= 0) goto err;
		if (X509_NAME_print_ex(bp,XPGP_get_uid_name(x),nmindent, nmflags) < 0) goto err;
		if (BIO_write(bp,"\n",1) <= 0) goto err;
		}
	if(!(cflag & XPGP_FLAG_NO_PUBKEY))
		{
		if (BIO_write(bp,"        Subject Public Key Info:\n",33) <= 0)
			goto err;
		if (BIO_printf(bp,"%12sPublic Key Algorithm: ","") <= 0)
			goto err;
		if (i2a_ASN1_OBJECT(bp, x->key->key->algor->algorithm) <= 0)
			goto err;
		if (BIO_puts(bp, "\n") <= 0)
			goto err;

		pkey=XPGP_PUBKEY_get_pubkey(x->key);
		if (pkey == NULL)
			{
			BIO_printf(bp,"%12sUnable to load Public Key\n","");
			ERR_print_errors(bp);
			}
		else
#ifndef OPENSSL_NO_RSA
		if (pkey->type == EVP_PKEY_RSA)
			{
			BIO_printf(bp,"%12sRSA Public Key: (%d bit)\n","",
			BN_num_bits(pkey->pkey.rsa->n));
			RSA_print(bp,pkey->pkey.rsa,16);
			}
		else
#endif
#ifndef OPENSSL_NO_DSA
		if (pkey->type == EVP_PKEY_DSA)
			{
			BIO_printf(bp,"%12sDSA Public Key:\n","");
			DSA_print(bp,pkey->pkey.dsa,16);
			}
		else
#endif
			BIO_printf(bp,"%12sUnknown Public Key:\n","");

		EVP_PKEY_free(pkey);
		}

	/* loop through the signatures */
	if (!x->signs)
		goto err;

	for(i = 0; i < sk_XPGP_SIGNATURE_num(x->signs); i++) 
	{
		BIO_printf(bp, "%6sSignature(%d):\n","",i);
		nsig = sk_XPGP_SIGNATURE_value(x->signs, i);
		if (!nsig) continue;
		if (!XPGP_SIGNATURE_print_ex(bp, nsig, nmflags, cflag))
			goto err;
	}

	ret=1;
err:
	return(ret);
	}


int XPGP_SIGNATURE_print_ex(BIO *bp, XPGP_SIGNATURE *x, unsigned long nmflags, unsigned long cflag)
	{
	long l;
	int nmindent = 0;
	int ret=0;

	if((nmflags & XN_FLAG_SEP_MASK) == XN_FLAG_SEP_MULTILINE) {
			nmindent = 12;
	}

	if(nmflags == X509_FLAG_COMPAT)
		nmindent = 16;

	if(!(cflag & XPGP_SIGNATURE_FLAG_NO_VERSION))
		{
		l=XPGP_SIGNATURE_get_version(x);
		if (BIO_printf(bp,"%8sVersion: %lu (0x%lx)\n","",l+1,l) <= 0) goto err;
		}

	if(!(cflag & XPGP_SIGNATURE_FLAG_NO_TYPE))
		{
		l=XPGP_SIGNATURE_get_type(x);
		if (BIO_printf(bp,"%8sType: (0x%lx)\n","",l) <= 0) goto err;
		}

	if(!(cflag & XPGP_SIGNATURE_FLAG_NO_TIMESTAMP))
		{
		if (BIO_write(bp,"        Timestamp: ",19) <= 0) goto err;
		if (!ASN1_TIME_print(bp,XPGP_SIGNATURE_get_timestamp(x))) 
			goto err;
		if (BIO_write(bp,"\n",1) <= 0) goto err;
		}

	if(!(cflag & XPGP_SIGNATURE_FLAG_NO_ISSUER))
		{
		if (BIO_printf(bp,"        Issuer: ") <= 0) goto err;
		if (X509_NAME_print_ex(bp,XPGP_SIGNATURE_get_issuer_name(x),nmindent, nmflags) < 0) goto err;
		if (BIO_write(bp,"\n",1) <= 0) goto err;
		}

/***
	if(!(cflag & XPGP_SIGNATURE_FLAG_NO_SIGNAME))
		{
		if (BIO_printf(bp,"%8sSignature Algorithm: ","") <= 0) 
			goto err;
		if (i2a_ASN1_OBJECT(bp, x->signtype->algorithm) <= 0)
			goto err;
		if (BIO_puts(bp, "\n") <= 0)
			goto err;
		}
***/

	if(!(cflag & XPGP_SIGNATURE_FLAG_NO_SIGDUMP))
		{
		if(X509_signature_print(bp, x->signtype, x->signature) <= 0) goto err;
		}


	ret=1;
err:
	return(ret);
	}


