/* crypto/xpgp/xPGP_set.c */
/*
 * Copyright (C) 2005-2006 Mark Fernie (drbob@lunamutt.com)  
 * All rights reserved.
 */
/* ====================================================================
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit. (http://www.openssl.org/)"
 *
 * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    openssl-core@openssl.org.
 *
 * 5. Products derived from this software may not be called "OpenSSL"
 *    nor may "OpenSSL" appear in their names without prior written
 *    permission of the OpenSSL Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit (http://www.openssl.org/)"
 *
 * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 *
 */


#include <stdio.h>
#include "cryptlib.h"
#include <openssl/asn1.h>
#include <openssl/objects.h>
#include <openssl/evp.h>
#include <openssl/xPGP.h>

/* A couple of get's as well...
 */

EVP_PKEY *XPGP_get_pubkey(XPGP *x)
	{
        if ((x == NULL) || (x->key == NULL))
                return(NULL);
        return(X509_PUBKEY_get(x->key->key));
        }

EVP_PKEY *XPGP_PUBKEY_get_pubkey(XPGP_PUBKEY *x)
	{
        if (x == NULL)
                return(NULL);
        return(X509_PUBKEY_get(x->key));
        }

X509_NAME *XPGP_get_uid_name(XPGP *a)
{
	        return(a->subject->subject);
}


X509_NAME *XPGP_SIGNATURE_get_issuer_name(XPGP_SIGNATURE *a)
{
	        return(a->issuer);
}


int XPGP_set_version(XPGP *x, long version)
	{
	if (x == NULL) return(0);
	if (x->version == NULL)
		{
		if ((x->version=M_ASN1_INTEGER_new()) == NULL)
			return(0);
		}
	return(ASN1_INTEGER_set(x->version,version));
	}


int XPGP_set_uid_name(XPGP *x, X509_NAME *name)
	{
	if ((x == NULL) || (x->subject == NULL)) return(0);
	return(X509_NAME_set(&x->subject->subject,name));
	}


// Set the variables on the PubKey.

int XPGP_PUBKEY_set_version(XPGP_PUBKEY *x, long version)
	{
	if (x == NULL) return(0);
	if (x->version == NULL)
		{
		if ((x->version=M_ASN1_INTEGER_new()) == NULL)
			return(0);
		}
	return(ASN1_INTEGER_set(x->version,version));
	}

int XPGP_PUBKEY_set_notBefore(XPGP_PUBKEY *x, ASN1_TIME *tm)
	{
	ASN1_TIME *in;

	if ((x == NULL) || (x->validity == NULL)) return(0);
	in=x->validity->notBefore;
	if (in != tm)
		{
		in=M_ASN1_TIME_dup(tm);
		if (in != NULL)
			{
			M_ASN1_TIME_free(x->validity->notBefore);
			x->validity->notBefore=in;
			}
		}
	return(in != NULL);
	}

int XPGP_PUBKEY_set_notAfter(XPGP_PUBKEY *x, ASN1_TIME *tm)
	{
	ASN1_TIME *in;

	if ((x == NULL) || (x->validity == NULL)) return(0);
	in=x->validity->notAfter;
	if (in != tm)
		{
		in=M_ASN1_TIME_dup(tm);
		if (in != NULL)
			{
			M_ASN1_TIME_free(x->validity->notAfter);
			x->validity->notAfter=in;
			}
		}
	return(in != NULL);
	}


int XPGP_PUBKEY_set_pubkey(XPGP_PUBKEY *x, EVP_PKEY *pkey)
	{
	if (x == NULL) return(0);
	return(X509_PUBKEY_set(&(x->key),pkey));
	}

