/* crypto/xpgp/xPGP_vfy.h */
/*
 * Copyright (C) 2005-2006 Mark Fernie (drbob@lunamutt.com)  
 * All rights reserved.
 */
/* ====================================================================
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit. (http://www.openssl.org/)"
 *
 * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    openssl-core@openssl.org.
 *
 * 5. Products derived from this software may not be called "OpenSSL"
 *    nor may "OpenSSL" appear in their names without prior written
 *    permission of the OpenSSL Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit (http://www.openssl.org/)"
 *
 * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 *
 */




#ifndef HEADER_XPGP_H
  #include <openssl/xPGP.h>
#endif

#ifndef HEADER_XPGP_VFY_H
#define HEADER_XPGP_VFY_H

#ifndef OPENSSL_NO_LHASH
#include <openssl/lhash.h>
#endif
#include <openssl/bio.h>
#include <openssl/crypto.h>
#include <openssl/symhacks.h>

#ifdef  __cplusplus
extern "C" {
#endif

/* xPGP equivalent to the store will be the keybox.
 *
 */

struct xPGP_keyring_st
{
	/* a list of certificates - grouped by trust level */
	int state; /* how organised */
	int error; /* last error message */
	int secLevel;

	int superNodeMode;

	XPGP *houseKeyPublic; /* your own */
	EVP_PKEY *houseKeyPrivate; /* your own */

	STACK_OF(XPGP) *trusted_signers;  /* we trust these peoples signatures */
	STACK_OF(XPGP) *trusted_ownsign;  /* we have signed these certificates */

	STACK_OF(XPGP) *trusted_certs;    /* enough points to id the person */
	STACK_OF(XPGP) *trusted_basic;    /* enough points to id the person */

	STACK_OF(XPGP) *untrusted_certs;  /* not enough signatures for full trust */
	STACK_OF(XPGP) *unknown_certs;    /* not signed by anyone I trust */
};


typedef struct xPGP_keyring_st XPGP_KEYRING;


#define TRUST_SIGN_OWN		6
#define TRUST_SIGN_TRSTED	5
#define TRUST_SIGN_AUTHEN	4
#define TRUST_SIGN_BASIC	3
#define TRUST_SIGN_UNTRUSTED	2
#define TRUST_SIGN_UNKNOWN      1
#define TRUST_SIGN_NONE		0
#define TRUST_SIGN_BAD		-1

	/* all the key fns that should be needed to be exposed */
int 	XPGP_add_certificate(XPGP_KEYRING *kr, XPGP *x);
int 	XPGP_remove_certificate(XPGP_KEYRING *kr, XPGP *x);
int 	XPGP_auth_certificate(XPGP_KEYRING *kr, XPGP *x);
int     XPGP_sign_certificate(XPGP_KEYRING *kr, XPGP *subj, XPGP *issuer);

/* required by ssl. 0 = failed, 1 = pass, */
int 	XPGP_verify_cert(XPGP_KEYRING *kr, XPGP *x);

int 	XPGP_check_valid_certificate(XPGP *x);

int     XPGP_signer_trusted(XPGP_KEYRING *kr, XPGP *trusted);
int     XPGP_signer_untrusted(XPGP_KEYRING *kr, XPGP *untrusted);

int 	XPGP_copy_known_signatures(XPGP_KEYRING *kr, XPGP *dest, XPGP *src);
XPGP *  XPGP_KEYRING_find_known_cert(XPGP_KEYRING *kr, XPGP *tofind);

int     XPGP_SIGNATURE_cmp(XPGP_SIGNATURE *s1, XPGP_SIGNATURE *s2);

 /**********************************************************************
  * High Level KeyRing Control Fns.
  **********************************************************************
  * 
  */

XPGP_KEYRING *createPGPContext(XPGP *own, EVP_PKEY *privkey);
int 	freePGPContext(XPGP_KEYRING *kr);

/* special fn that should have more controls on it 
 * This allows unsigned (but) valid XPGP to be accepted
 */

int 	XPGP_supernode(XPGP_KEYRING *kr); 


int     setHouseKey(XPGP_KEYRING *kr, XPGP *own, EVP_PKEY *privkey);
int     addCertificate(XPGP_KEYRING *kr, XPGP *unknown);
int     removeCertificate(XPGP_KEYRING *, XPGP *unwanted);
int     checkKeyRing(XPGP_KEYRING *);
int 	updateUntrustedStacks(XPGP_KEYRING *);

	/* handling the certificate.
	 * 
	 * control fns from the 
	 */

int 	XPGP_verify_cert(XPGP_KEYRING *kr, XPGP *x);

 /**********************************************************************
  * Signature related Fns.
  **********************************************************************
  *
  */

int	checkSignature(XPGP *candidate, XPGP *validator);
int	validateSignature(XPGP *candidate, XPGP_SIGNATURE *signature, XPGP *validator);

int	signCertificate(XPGP_KEYRING *kr, XPGP *subj, XPGP *issuer);
int 	findSignature(XPGP *subj, XPGP *issuer, XPGP_SIGNATURE **sign);


 /**********************************************************************
  * Certificate Moving related Fns.
  **********************************************************************
  *
  */

int 	existsCertificate(XPGP_KEYRING *kr, XPGP *key);
int 	addCertificateToStack(STACK_OF(XPGP) *sk, XPGP *newbie);
int 	removeCertificateFromStack(STACK_OF(XPGP) *sk, XPGP *unwanted);
int 	existsCertificateInStack(STACK_OF(XPGP) *sk, XPGP *toBorNot2B);
int 	moveStackToStack(STACK_OF(XPGP) *dest, STACK_OF(XPGP) *src);


 /**********************************************************************
  * Validation Fns.
  **********************************************************************
  *
  */

int validateOrClearCertifateStack(STACK_OF(XPGP) *);
int validateSelfSignedCertificate(XPGP *cert);
int checkOrClearStackSignedByKey(STACK_OF(XPGP) *sk, XPGP *pgp);
int validateCertificateIsSignedByKey(XPGP *subj, XPGP *issuer);

int checkStackTrustworthiness(XPGP_KEYRING *kr, STACK_OF(XPGP) *sk, int skLvl);
int moveToStack(XPGP_KEYRING *kr, XPGP *xpgp, int skLvl);

 /**********************************************************************
  * Trust Calculations +  Control Fns.
  **********************************************************************
  *
  */

 int     setSecurityLevel(XPGP_KEYRING *, int val);
 int     getTrustLevel(XPGP_KEYRING *, XPGP *peer);
 int	   getTrustSignatureLevel(XPGP_KEYRING *, XPGP *candidate, 
 				XPGP_SIGNATURE *sign);
 XPGP   *findSigner(STACK_OF(XPGP) *sk, XPGP_SIGNATURE *sign);
 int	    XPGP_SIGNER_cmp(XPGP *issuer, XPGP_SIGNATURE *sign);
 int     validateCertificate(XPGP_KEYRING *ikr, XPGP *peer);

int     DisplayStacks(XPGP_KEYRING *kr);
int     DisplayStack(STACK_OF(XPGP) *sk);
int     DisplayStackTrust(XPGP_KEYRING *kr);

#ifdef  __cplusplus
}
#endif
#endif
