/* crypto/xpgp/xPGPtype.c */
/*
 * Copyright (C) 2005-2006 Mark Fernie (drbob@lunamutt.com)  
 * All rights reserved.
 */
/* ====================================================================
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit. (http://www.openssl.org/)"
 *
 * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    openssl-core@openssl.org.
 *
 * 5. Products derived from this software may not be called "OpenSSL"
 *    nor may "OpenSSL" appear in their names without prior written
 *    permission of the OpenSSL Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit (http://www.openssl.org/)"
 *
 * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 *
 */


#include <stdio.h>
#include "cryptlib.h"
#include <openssl/evp.h>
#include <openssl/objects.h>
#include <openssl/xPGP.h>

int XPGP_certificate_type(XPGP *x, EVP_PKEY *pkey)
{
	EVP_PKEY *pk;
	int ret=0,i;

	if (x == NULL) return(0);

	if (pkey == NULL)
		pk=XPGP_get_pubkey(x);
	else
		pk=pkey;

	if (pk == NULL) return(0);

	switch (pk->type)
		{
	case EVP_PKEY_RSA:
		ret=EVP_PK_RSA|EVP_PKT_SIGN;
/*		if (!sign only extension) */
			ret|=EVP_PKT_ENC;
	break;
	case EVP_PKEY_DSA:
		ret=EVP_PK_DSA|EVP_PKT_SIGN;
		break;
	case EVP_PKEY_DH:
		ret=EVP_PK_DH|EVP_PKT_EXCH;
		break;
	default:
		break;
		}

	i=XPGP_get_signature_type(x);
	switch (i)
		{
	case EVP_PKEY_RSA:
		ret|=EVP_PKS_RSA;
		break;
	case EVP_PKEY_DSA:
		ret|=EVP_PKS_DSA;
		break;
	default:
		break;
		}

	if (EVP_PKEY_size(pk) <= 1024/8)/* /8 because it's 1024 bits we look
					   for, not bytes */
		ret|=EVP_PKT_EXP;
	if(pkey==NULL) EVP_PKEY_free(pk);
	return(ret);
}



int XPGP_get_signature_type(XPGP *x)
{
	if (sk_XPGP_SIGNATURE_num(x->signs) < 1)
	{
		/* the first one should provide the selfsignature */
		XPGP_SIGNATURE *ss = sk_XPGP_SIGNATURE_value(x->signs,0);
		return EVP_PKEY_type(OBJ_obj2nid(ss->signtype->algorithm));
	}
	/* error */
	return -1;
}



