/* crypto/xpgp/xPGP.h */
/*
 * Copyright (C) 2005-2006 Mark Fernie (drbob@lunamutt.com)  
 * All rights reserved.
 */
/* ====================================================================
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit. (http://www.openssl.org/)"
 *
 * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    openssl-core@openssl.org.
 *
 * 5. Products derived from this software may not be called "OpenSSL"
 *    nor may "OpenSSL" appear in their names without prior written
 *    permission of the OpenSSL Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit (http://www.openssl.org/)"
 *
 * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 *
 */

#ifndef HEADER_XPGP_H
#define HEADER_XPGP_H

#include <openssl/symhacks.h>
#ifndef OPENSSL_NO_BUFFER
#include <openssl/buffer.h>
#endif
#ifndef OPENSSL_NO_EVP
#include <openssl/evp.h>
#endif
#ifndef OPENSSL_NO_BIO
#include <openssl/bio.h>
#endif
#include <openssl/stack.h>
#include <openssl/asn1.h>
#include <openssl/safestack.h>

#ifndef OPENSSL_NO_RSA
#include <openssl/rsa.h>
#endif

#ifndef OPENSSL_NO_DSA
#include <openssl/dsa.h>
#endif

#ifndef OPENSSL_NO_DH
#include <openssl/dh.h>
#endif
#ifndef OPENSSL_NO_SHA
#include <openssl/sha.h>
#endif
#include <openssl/e_os2.h>
#include <openssl/ossl_typ.h>

#include <openssl/x509.h>

#ifdef  __cplusplus
extern "C" {
#endif


typedef struct xpgp_uid_st
{
	X509_NAME *subject;
} XPGP_UID;

// don't think these are needed.
//DECLARE_STACK_OF(XPGP_UID)
//DECLARE_ASN1_SET_OF(XPGP_UID)

typedef struct xpgp_pubkey_st
{
	ASN1_INTEGER *version;
	X509_VAL *validity;
// Already in X509_PUBKEY....	X509_ALGOR *keytype;
// Shouldn't be here.	ASN1_OCTET_STRING *keyid; // 64 bit.
	X509_PUBKEY *key;
} XPGP_PUBKEY;

// don't think these are needed.
//DECLARE_STACK_OF(XPGP_PUBKEY)
//DECLARE_ASN1_SET_OF(XPGP_PUBKEY)

// Flags are below....
typedef struct xpgp_trust_st
{
	ASN1_INTEGER *trust_lvl;
} XPGP_TRUST_OBJ;

DECLARE_STACK_OF(XPGP_TRUST_OBJ)
DECLARE_ASN1_SET_OF(XPGP_TRUST_OBJ)

typedef struct xpgp_signature_st
{
	// version + timestamp
	ASN1_INTEGER *version;
	ASN1_INTEGER *type;      // sign or recovation
	ASN1_TIME    *timestamp; // point of creation

	// Signer details.
	X509_NAME *issuer;
	ASN1_OCTET_STRING *keyid; // 64 bit.

	// signature.
	X509_ALGOR *signtype;
	ASN1_BIT_STRING *signature;

	// Not included in the Export.
	XPGP_TRUST_OBJ *trust_signer;
	int references;
} XPGP_SIGNATURE;

// STACK NEEDED
DECLARE_STACK_OF(XPGP_SIGNATURE)
DECLARE_ASN1_SET_OF(XPGP_SIGNATURE)

// This is the Block that is being passed around....
// between clients... though  will have to reset Trust flags...
typedef struct xpgp_certificate_st
//struct xPGP_st
{
	ASN1_INTEGER *version;
	XPGP_PUBKEY *key;
	XPGP_UID    *subject;
	STACK_OF(XPGP_SIGNATURE) *signs;

	XPGP_TRUST_OBJ *trust_to_sign;
	XPGP_TRUST_OBJ *trust_cert;

	char *name;
	int valid;
	int references;

//} /* XPGP */ ;
} XPGP_CERT;

typedef XPGP_CERT XPGP;

DECLARE_STACK_OF(XPGP)
DECLARE_ASN1_SET_OF(XPGP)

/* A Special ASN.1 Structure which is used for
 * the signing of the data.....
 * This function is temporarily created
 * and used for this stuff.
 */

typedef struct xpgp_signdata_st
{
	ASN1_INTEGER *version;
	ASN1_INTEGER *type;      // sign or recovation
	ASN1_TIME    *timestamp;

	X509_VAL *validity;
	X509_PUBKEY *key;
	X509_NAME *subject;

	X509_NAME *issuer;
	X509_ALGOR *signtype;
} XPGP_SIGNDATA;



DECLARE_STACK_OF(XPGP_SIGNDATA)
DECLARE_ASN1_SET_OF(XPGP_SIGNDATA)

#define XPGP_TRUST_KEY		0x01
#define XPGP_TRUST_UID		0x02
#define XPGP_TRUST_SIG		0x04

// only for == comparisions.
#define XPGP_TRUST_KEY_UNKNOWN		0b001
#define XPGP_TRUST_KEY_NOT_TO_SIGN	0b010
#define XPGP_TRUST_KEY_MARGIN_SIGN	0b101
#define XPGP_TRUST_KEY_TRUST_SIGN	0b110
#define XPGP_TRUST_KEY_OUR_KEY		0b111

// OR/AND ed
#define XPGP_TRUST_KEY_DISABLED		(1 >> 5)
#define XPGP_TRUST_KEY_BUCKSTOP		(1 >> 7)

// only for == comparisions.
#define XPGP_TRUST_UID_UNKNOWN		0b00
#define XPGP_TRUST_UID_NOT		0b01
#define XPGP_TRUST_UID_MARGINAL		0b10
#define XPGP_TRUST_UID_COMPLETE		0b11

// OR/AND ed
#define XPGP_TRUST_UID_WARNONLY		(1 >> 7)

// only for == comparisions.
#define XPGP_TRUST_SIG_UNKNOWN		0b001
#define XPGP_TRUST_SIG_NOT		0b010
#define XPGP_TRUST_SIG_MARGINAL		0b101
#define XPGP_TRUST_SIG_COMPLETE		0b110
#define XPGP_TRUST_SIG_ULTIMATE		0b111

// OR/AND ed
#define XPGP_TRUST_SIG_CHECKED		(1 >> 6)
#define XPGP_TRUST_SIG_CONTIG		(1 >> 7)

// ASN1_FUNCTIONS.
DECLARE_ASN1_FUNCTIONS(XPGP_UID)

DECLARE_ASN1_FUNCTIONS(XPGP_PUBKEY)

DECLARE_ASN1_FUNCTIONS(XPGP_SIGNATURE)

DECLARE_ASN1_FUNCTIONS(XPGP_TRUST_OBJ)

DECLARE_ASN1_FUNCTIONS(XPGP)

DECLARE_ASN1_FUNCTIONS(XPGP_SIGNDATA)

// Define the functions to access the data via....

#ifdef SSLEAY_MACROS

#define d2i_XPGP_fp(fp,xpgp) (XPGP *)ASN1_d2i_fp((char *(*)())XPGP_new, \
	(char *(*)())d2i_XPGP, (fp),(unsigned char **)(xpgp))
#define i2d_XPGP_fp(fp,xpgp) ASN1_i2d_fp(i2d_XPGP,fp,(unsigned char *)xpgp)

#define d2i_XPGP_bio(bp,xpgp) (XPGP *)ASN1_d2i_bio((char *(*)())XPGP_new, \
	(char *(*)())d2i_XPGP, (bp),(unsigned char **)(xpgp))
#define i2d_XPGP_bio(bp,xpgp) ASN1_i2d_bio(i2d_XPGP,bp,(unsigned char *)xpgp)

#endif


// declarations of the functions...

#ifndef OPENSSL_NO_FP_API

XPGP *d2i_XPGP_fp(FILE *fp, XPGP **xpgp);
int i2d_XPGP_fp(FILE *fp,XPGP *xpgp);

#endif

#ifndef OPENSSL_NO_BIO
XPGP *d2i_XPGP_bio(BIO *bp, XPGP **xpgp);
int i2d_XPGP_bio(BIO *bp,XPGP *xpgp);
#endif


/* Flags for XPGP_print_ex */

#define XPGP_FLAG_NO_HEADER    			1L
#define XPGP_FLAG_NO_VERSION			(1L << 1)	
#define XPGP_FLAG_NO_VALIDITY			(1L << 2)
#define XPGP_FLAG_NO_SUBJECT			(1L << 3)
#define XPGP_FLAG_NO_PUBKEY			(1L << 4)

/* Flags for XPGP_SIGNATURE_print_ex */

#define XPGP_SIGNATURE_FLAG_NO_VERSION		(1L << 8)
#define XPGP_SIGNATURE_FLAG_NO_TYPE		(1L << 9)
#define XPGP_SIGNATURE_FLAG_NO_TIMESTAMP	(1L << 10)
#define XPGP_SIGNATURE_FLAG_NO_ISSUER		(1L << 11)
#define XPGP_SIGNATURE_FLAG_NO_SIGNAME		(1L << 12)
#define XPGP_SIGNATURE_FLAG_NO_SIGDUMP		(1L << 13)


/* Printing Fns */
int             XPGP_print(BIO *bp,XPGP *x);
int             XPGP_print_ex(BIO *bp,XPGP *x, 
			unsigned long nmflag, unsigned long cflag);

int             XPGP_print_fp(FILE *fp,XPGP *x);
int             XPGP_print_ex_fp(FILE *fp,XPGP *x, 
			unsigned long nmflag, unsigned long cflag);

int             XPGP_SIGNATURE_print_ex(BIO *bp,XPGP_SIGNATURE *x,
			unsigned long nmflag, unsigned long cflag);


// Get Fns.
/* long		XPGP_get_version(XPGP *x); 
 * ASN1_TIME *	XPGP_PUBKEY_get_notBefore(XPGP_PUBKEY *x);
 * ASN1_TIME *	XPGP_PUBKEY_get_notAfter(XPGP_PUBKEY *x);
 * long 	XPGP_SIGNATURE_get_version(XPGP_SIGNATURE *x);
 * long 	XPGP_SIGNATURE_get_type(XPGP_SIGNATURE *x);
 * ASN1_TIME *	XPGP_SIGNATURE_get_timestamp(XPGP_SIGNATURE *x);
 */

#define		XPGP_get_version(x) ASN1_INTEGER_get((x)->version)
#define		XPGP_PUBKEY_get_notBefore(x) ((x)->validity->notBefore)
#define		XPGP_PUBKEY_get_notAfter(x)  ((x)->validity->notAfter)
#define 	XPGP_SIGNATURE_get_version(x) ASN1_INTEGER_get((x)->version)
#define 	XPGP_SIGNATURE_get_type(x) ASN1_INTEGER_get((x)->type)
#define		XPGP_SIGNATURE_get_timestamp(x) ((x)->timestamp)

// actual fns (in xPGP_set.c)
X509_NAME *	XPGP_get_uid_name(XPGP *x);
EVP_PKEY *	XPGP_get_pubkey(XPGP *x);
EVP_PKEY *	XPGP_PUBKEY_get_pubkey(XPGP_PUBKEY *x);
X509_NAME *	XPGP_SIGNATURE_get_issuer_name(XPGP_SIGNATURE *x);


// Set Fns.
int 		XPGP_set_version(XPGP *x, long version);
int 		XPGP_set_uid_name(XPGP *x, X509_NAME *name);

int 		XPGP_PUBKEY_set_version(XPGP_PUBKEY *x, long version);
int 		XPGP_PUBKEY_set_notBefore(XPGP_PUBKEY *x, ASN1_TIME *tm);
int 		XPGP_PUBKEY_set_notAfter(XPGP_PUBKEY *x, ASN1_TIME *tm);
int 		XPGP_PUBKEY_set_pubkey(XPGP_PUBKEY *x, EVP_PKEY *pkey);


/* High Level Sign Function */
int XPGP_verify_it(XPGP *subj, XPGP_SIGNATURE *sign, EVP_PKEY *pkey);
int XPGP_sign_it(XPGP *subj, XPGP *issuer, EVP_PKEY *pkey, const EVP_MD *md);

/* Lower Helper Fns */
int XPGP_setup_data_sign(XPGP_SIGNDATA *d, XPGP *subj, XPGP *issuer);
int XPGP_setup_data_verify(XPGP_SIGNDATA *d, XPGP *x, XPGP_SIGNATURE *s);
int XPGP_setup_signature(XPGP_SIGNATURE *sign, XPGP_SIGNDATA *data, XPGP *issuer);
int XPGP_add_signature(XPGP *subj, XPGP_SIGNATURE *sign);

 /******************************************************************
  * xPGP_cmp.c
  ******************************************************************
  *
  */
int XPGP_check_private_key(XPGP *x, EVP_PKEY *k);
int XPGP_cmp(const XPGP *a, const XPGP *b);


 /******************************************************************
  * xPGPtype.c
  ******************************************************************
  *
  */

int XPGP_get_signature_type(XPGP *x);
int XPGP_certificate_type(XPGP *x,EVP_PKEY *pubkey /* optional */);


/* INCLUDE the KEYRING and verification fns. */
#include <openssl/xPGP_vfy.h>

#ifndef SSLEAY_MACROS

//int XPGP_sign(XPGP_SIGNATURE *sign, XPGP_SIGNDATA *data,
//		EVP_PKEY *pkey,const EVP_MD *md);
//int X509_verify(X509 *a, EVP_PKEY *r);
//int XPGP_verify(XPGP_SIGNATURE *s, XPGP_SIGNDATA *d, EVP_PKEY *pubkey);

#endif

#ifdef SSLEAY_MACROS

/* We sign using the following functions.
 * int ASN1_sign(
 *		int (*i2d)(), // fn to convert internal to signing block.
 *		X509_ALGOR *algor1, // sign alg 1 (can be null)
 *		X509_ALGOR *algor2, // sign alg 2 (preferental)
 *		ASN1_BIT_STRING *signature, // output buffer.
 *		char *data,     // data ptr to pass to the i2d fn.
 *		EVP_PKEY *pkey, // key to sign with.
 *		const EVP_MD *type) // hash type.
 *
 * modelled on:
 *
 * #define X509_sign(x,pkey,md) \
 *	ASN1_sign((int (*)())i2d_X509_CINF, x->cert_info->signature, \
 *      x->sig_alg, x->signature, (char *)x->cert_info,pkey,md)
 */
#endif

#define XPGP_sign(s,d,pkey,md) \
	ASN1_sign((int (*)())i2d_XPGP_SIGNDATA, d->signtype, \
        s->signtype, s->signature, (char *) d, pkey, md)

#define XPGP_verify(s,d,r) ASN1_verify((int (*)())i2d_XPGP_SIGNDATA,\
	s->signtype, s->signature, (char *) d, r)



/*
 * PEM stuff.
 *
 * (defined here, to minimise changes to system)
 *
 */

#include <openssl/pem.h>

#define PEM_STRING_XPGP         "XPGP CERTIFICATE"


/* In the final version the functions below need to be
 * redefined properly (inside and outside of the SSLEAY_MACROS
 */

DECLARE_PEM_rw(XPGP, XPGP)


#if 0

#define PEM_write_XPGP(fp,x) \
	PEM_ASN1_write((int (*)())i2d_XPGP,PEM_STRING_XPGP,fp, \
	        (char *)x, NULL,NULL,0,NULL,NULL)

#define PEM_read_XPGP(fp,x,cb,u) (XPGP *)PEM_ASN1_read( \
        (char *(*)())d2i_XPGP,PEM_STRING_XPGP,fp,(char **)x,cb,u)

#endif




 /******************************************************************
  * xPGP_aux.c
  ******************************************************************
  *
  */

int generate_xpgp(
	const char *pgp_file, const char *pk_file, const char *passwd,
	const char *name, const char *email, const char *org,
	const char *loc, const char *state, const char *country, int nbits);
	
void generate_XPGP(XPGP **pgp, EVP_PKEY **pkey,
	const char *name, const char *email, const char *org,
	const char *loc, const char *state, const char *country, int nbits);
	
	
void sign_xpgp(const char *subj_file, const char *issuer_file,
	const char *ipk_file, const char *ipk_passwd);
	
void sign_XPGP(XPGP *subj, XPGP *issuer, EVP_PKEY *privKey);
	
XPGP *load_xpgp(const char *file);
EVP_PKEY *load_privKey(const char *pk_file, const char *pk_passwd);

XPGP_KEYRING *xpgp_init(const char *pgp_file, const char *pk_file, const char *pk_passwd);

void display_xpgp(const char *fname);
void display_XPGP(XPGP *pgp, FILE *out);

int xpgp_display_block(const unsigned char *data, const unsigned int len, 
	const unsigned char *title);


#ifdef  __cplusplus
}
#endif


#endif

