/* ssl/tls1.h */
/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
 * All rights reserved.
 *
 * This package is an SSL implementation written
 * by Eric Young (eay@cryptsoft.com).
 * The implementation was written so as to conform with Netscapes SSL.
 * 
 * This library is free for commercial and non-commercial use as long as
 * the following conditions are aheared to.  The following conditions
 * apply to all code found in this distribution, be it the RC4, RSA,
 * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
 * included with this distribution is covered by the same copyright terms
 * except that the holder is Tim Hudson (tjh@cryptsoft.com).
 * 
 * Copyright remains Eric Young's, and as such any Copyright notices in
 * the code are not to be removed.
 * If this package is used in a product, Eric Young should be given attribution
 * as the author of the parts of the library used.
 * This can be in the form of a textual message at program startup or
 * in documentation (online or textual) provided with the package.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    "This product includes cryptographic software written by
 *     Eric Young (eay@cryptsoft.com)"
 *    The word 'cryptographic' can be left out if the rouines from the library
 *    being used are not cryptographic related :-).
 * 4. If you include any Windows specific code (or a derivative thereof) from 
 *    the apps directory (application code) you must include an acknowledgement:
 *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
 * 
 * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 */

#ifndef HEADER_PGP1_H 
#define HEADER_PGP1_H 

#include <openssl/buffer.h>
//#include "ssl_locl.h"

#include "tls1.h"

#ifdef  __cplusplus
extern "C" {
#endif

/* Temp tst fns */
int pgp1_checkMemLeaks(); /* check mem leakage */

// rmf24 - using TLS codes ..... EXCEPT FOR VERSIONING.
#define PGP1_VERSION			0x0701
#define PGP1_VERSION_MAJOR		0x07
#define PGP1_VERSION_MINOR		0x01

	// pgp_meth.c
SSL_METHOD *PGPv1_method(void);

	// pgp_lib.c
XPGP *ssl_get_server_send_pgp_cert(SSL *s);
XPGP *SSL_get_peer_pgp_certificate(const SSL *s);
int SSL_CTX_check_pgp_private_key(const SSL_CTX *ctx);
int SSL_check_pgp_private_key(const SSL *ssl);

	// set the Keyring to the context.
int SSL_CTX_set_XPGP_KEYRING(SSL_CTX *ctx, XPGP_KEYRING *kr);


	// pgp1_lib.c
SSL_METHOD *pgpv1_base_method(void);

	// pgp_srvr.c
int pgp1_accept(SSL *s);
SSL_METHOD *PGPv1_server_method(void);
int pgp1_send_server_certificate(SSL *s);

	// defined by wrappers in ssl3_srvr.c
int pgp1_send_hello_request(SSL *s);
int pgp1_get_client_hello(SSL *s);
int pgp1_check_client_hello(SSL *s);
int pgp1_send_server_hello(SSL *s);
int pgp1_send_server_done(SSL *s);
int pgp1_send_server_key_exchange(SSL *s);
int pgp1_get_client_key_exchange(SSL *s);


	// pgp_clnt.c
int pgp1_connect(SSL *s);
SSL_METHOD *PGPv1_client_method(void);

	// defined by wrappers in ssl2_clnt.c
int pgp1_client_hello(SSL *s);
int pgp1_get_server_hello(SSL *s);
int pgp1_get_server_done(SSL *s);
int pgp1_send_client_verify(SSL *s);

// in pgp1_both.c
int ssl_pgp_cert_type(XPGP *x, EVP_PKEY *pkey);
int pgp1_send_change_cipher_spec(SSL *s, int a, int b);
unsigned long pgp1_output_pgp_cert(SSL *s, XPGP *x);

// in pgp1_cert.c
int ssl_verify_pgp_cert(SSL *s, XPGP *x);

// in pgp_rsa.c
int SSL_use_pgp_certificate(SSL *ssl, XPGP *x);
int SSL_use_pgp_certificate_file(SSL *ssl, const char *file, int type);

int SSL_use_pgp_RSAPrivateKey(SSL *ssl, RSA *rsa);
int SSL_use_pgp_RSAPrivateKey_file(SSL *ssl, const char *file, int type);
int SSL_use_pgp_RSAPrivateKey_ASN1(SSL *ssl, unsigned char *d, long len);

int SSL_use_pgp_PrivateKey(SSL *ssl, EVP_PKEY *pkey);
int SSL_use_pgp_PrivateKey_file(SSL *ssl, const char *file, int type);
int SSL_use_pgp_PrivateKey_ASN1(int type, SSL *ssl, unsigned char *d, long len);

int SSL_CTX_use_pgp_certificate(SSL_CTX *ctx, XPGP *x);
int SSL_CTX_use_pgp_certificate_file(SSL_CTX *ctx, const char *file, int type);
int SSL_CTX_use_pgp_certificate_ASN1(SSL_CTX *ctx, int len, unsigned char *d);

int SSL_CTX_use_pgp_RSAPrivateKey(SSL_CTX *ctx, RSA *rsa);
int SSL_CTX_use_pgp_RSAPrivateKey_file(SSL_CTX *ctx, const char *file, int type);
int SSL_CTX_use_pgp_RSAPrivateKey_ASN1(SSL_CTX *ctx, unsigned char *d, long len);

int SSL_CTX_use_pgp_PrivateKey(SSL_CTX *ctx, EVP_PKEY *pkey);
int SSL_CTX_use_pgp_PrivateKey_file(SSL_CTX *ctx, const char *file, int type);
int SSL_CTX_use_pgp_PrivateKey_ASN1(int type, SSL_CTX *ctx, unsigned char *d, long);

// Functions required from elsewhere (xPGP/etc)
// These should have correct signatures.
//EVP_PKEY *XPGP_get_pubkey(XPGP *x);
//int XPGP_get_signature_type(XPGP *x);


#ifdef  __cplusplus
}
#endif


#endif

